package com.huimv.receive.controller;


import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.receive.common.utils.*;
import com.huimv.receive.entity.SysAccountMultilevel;
import com.huimv.receive.entity.SysUser;
import com.huimv.receive.service.ISysAccountMultilevelService;
import com.huimv.receive.service.ISysUserService;
import com.huimv.receive.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/admin/user")
@CrossOrigin
public class SysUserController {
    @Autowired
    private IUserService iUserService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysAccountMultilevelService accountMultilevelService;


//    @PostMapping("/list")
//    public JSONObject listUser(@RequestBody Map map) {
//        System.out.println();
//        int pageNum = (Integer)map.get("pageNum");
//        int pageSize = (Integer)map.get("pageSize");
//        String searchStr = (String)map.get("searchStr");
//        return iUserService.findUserAccount(searchStr,pageNum,pageSize);
//    }
    //员工分页查询
    @PostMapping("/page")
    public Result page(@RequestBody Map map) {
        return sysUserService.pageAll(map);
    }
    //员工查询
    @PostMapping("/list")
    public Result list(@RequestBody Map map) {
        return sysUserService.list(map);
    }

    //员工新增
    @PostMapping("/add")
    public Result add(@RequestBody SysUser sysUser) {
        String cardType = sysUser.getCardType();
        String cardNum = sysUser.getCardNum();
        if ("身份证".equals(cardType) && !IDCardValidator.isValidIDCard(cardNum))
            return new Result(10001,"身份证格式不正确",false);

        String userPhone = sysUser.getUserPhone();
        if (!PhoneNumberValidator.isValidPhoneNumber(userPhone))
            return new Result(10001,"手机号格式不正确",false);

        int count = sysUserService.count(new QueryWrapper<SysUser>().eq("card_type", cardType).eq("card_num", cardNum));
        if (count >0)
            return new Result(10001,"身份证已存在",false);

        int countUserPhone = sysUserService.count(new QueryWrapper<SysUser>().eq("user_phone", userPhone));
        if (countUserPhone >0)
            return new Result(10001,"身份证已存在",false);
        if (ObjectUtil.isNotEmpty(sysUser.getBirthday())){
            sysUser.setAge(DateUtil.ageOfNow(sysUser.getBirthday()));
        }
        sysUserService.save(sysUser);
        return new Result(10000,"保存成功",true);
    }

    //查询工作地点
    @PostMapping("/listWorkLocation")
    public Result listWorkLocation(@RequestBody Map map) {
        return sysUserService.listWorkLocation(map);
    }

    @PostMapping("/update")
    public Result update(@RequestBody SysUser sysUser) {
        String cardType = sysUser.getCardType();
        String cardNum = sysUser.getCardNum();
        if ("身份证".equals(cardType) && !IDCardValidator.isValidIDCard(cardNum))
            return new Result(10001,"身份证格式不正确",false);

        String userPhone = sysUser.getUserPhone();
        if (!PhoneNumberValidator.isValidPhoneNumber(userPhone))
            return new Result(10001,"手机号格式不正确",false);

        int count = sysUserService.count(new QueryWrapper<SysUser>().eq("card_type", cardType).eq("card_num", cardNum));
        if (count >1)
            return new Result(10001,"身份证已存在",false);

        int countUserPhone = sysUserService.count(new QueryWrapper<SysUser>().eq("user_phone", userPhone));
        if (countUserPhone >1)
            return new Result(10001,"身份证已存在",false);
        if (ObjectUtil.isNotEmpty(sysUser.getBirthday())){
            sysUser.setAge(DateUtil.ageOfNow(sysUser.getBirthday()));
        }
        sysUserService.save(sysUser);
        return new Result(10000,"保存成功",true);
    }

    @PostMapping("/remove")
    @Transactional
    public Result remove(@RequestBody Map<String,String> map) {
        String ids = map.get("ids");
        if (StringUtils.isBlank(ids))
            return new Result(10001,"所选用户不存在",false);
        List<String> collection = Arrays.stream(ids.split(","))
                .collect(Collectors.toList());
         sysUserService.removeByIds(collection);
         //删除账号
        accountMultilevelService.remove(new QueryWrapper<SysAccountMultilevel>().in("user_id",collection));
        return new Result(10000,"删除成功",true);
    }

    //账号查询
    @PostMapping("/pageAccount")
    public Result pageAccount(@RequestBody Map<String,Object> map) {
        return  sysUserService.pageAccount(map);
    }



    //账号增加
    @PostMapping("/addAccount")
    public Result addAccount(@RequestBody Map<String,Object> map) {
        return  sysUserService.addAccount(map);
    }

    //账号修改
    @PostMapping("/updateAccount")
    public Result updateAccount(@RequestBody Map<String,Object> map) {
        return  sysUserService.updateAccount(map);
    }

    @GetMapping("/printUser")
    public void printUser(HttpServletResponse response, @RequestParam(name = "ids") String ids) throws Exception {
        Map<String, String> map = new HashMap<>();
        map.put("ids", ids);
        sysUserService.printUser(response, map);
        String path = "/opt/huatong/user.xls";
        PdfUtil.returnPdfStream3(response, path, "人员列表");
    }
    //账号删除
    @PostMapping("/removeAccount")
    @Transactional
    public Result removeAccount(@RequestBody Map<String,String> map) {
        return  sysUserService.removeAccount(map);
    }

    //性别年龄分布
    @PostMapping("/userAgeAndGender")
    public Result userAgeAndGender(@RequestBody Map<String,String> map) {
        return  sysUserService.userAgeAndGender(map);
    }

    //岗位分布
    @PostMapping("/userJob")
    public Result userJob(@RequestBody Map<String,String> map) {
        return  sysUserService.userJob(map);
    }

}
